@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2001-2006 the xine project
+ * Copyright (C) 2001-2007 the xine project
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -15,8 +15,6 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
  *
- * $Id: gtkvideo.c,v 1.115 2006/04/04 22:08:16 dsalt Exp $
- *
  * gtk xine video widget
  *
  * some code originating from totem's gtkxine widget
@@ -55,6 +53,10 @@
 #include <X11/extensions/Xinerama.h>
 #endif
 
+#ifdef HAVE_XCB
+#include <xcb/xcb.h>
+#endif
+
 #ifdef WITH_DBUS
 #include <dbus/dbus-glib.h>
 
@@ -74,6 +76,8 @@
 #include <gtk/gtkmain.h>
 #include <gdk/gdkx.h>
 #include <xine.h>
+/* for get_capabilities() */
+#include <xine/video_out.h>
 
 #include "gtkvideo.h"
 #include "globals.h"
@@ -102,6 +106,7 @@ static void gtk_video_size_allocate (GtkWidget      *widget,
  				     GtkAllocation  *allocation);
 
 static gboolean gtv_unblank_screen (GtkVideo *);
+static gboolean gtv_update_resize_factor (GtkVideo *);
 
 static GtkWidgetClass *parent_class = NULL;
 static pthread_mutex_t resize_lock = PTHREAD_MUTEX_INITIALIZER;
@@ -110,13 +115,15 @@ static pthread_mutex_t resize_add_lock = PTHREAD_MUTEX_INITIALIZER;
 #define RESIZE_LOCK() pthread_mutex_lock (&resize_lock)
 #define RESIZE_UNLOCK() pthread_mutex_unlock (&resize_lock)
 
-#define RESIZE_ADD() \
+#define RESIZE_ADD(resize) \
   do { \
     pthread_mutex_lock (&resize_add_lock); \
     if (!gtv->priv->resizing && GTK_WIDGET_REALIZED (&gtv->widget)) \
       gtv->priv->resizing = \
 	g_idle_add_full (G_PRIORITY_HIGH_IDLE, \
-			 (GSourceFunc) gtk_video_idle_resize, gtv, NULL); \
+			 (GSourceFunc) ((resize) \
+			   ? gtk_video_idle_resize : gtk_video_idle_no_resize), \
+			 gtv, NULL); \
     pthread_mutex_unlock (&resize_add_lock); \
   } while (0)
 
@@ -124,6 +131,8 @@ static pthread_mutex_t resize_add_lock = PTHREAD_MUTEX_INITIALIZER;
  * private data
  */
 
+typedef void (*frame_cb_t) (gpointer);
+
 struct gtk_video_private_s {
 
   xine_t                  *xine;
@@ -177,18 +186,28 @@ struct gtk_video_private_s {
 #endif
 
   gboolean		 playing_no_blank:8;
-  /* visualization */
+
+  /* visualisation & post-plugins*/
+  gboolean		   have_tvtime:8;
+
   gboolean		   vis_active:8;
   xine_post_t		  *vis_plugin;
   char			  *vis_plugin_id;
   xine_audio_port_t       *vis_audio;
 
-  /* post-plugins */
   struct {
     GList	*list;
     gboolean	 enable;
   }			   post_deinterlace, post_video, post_audio;
-  gboolean		   have_tvtime;
+
+  /* callbacks */
+
+  guint			   frame_cb_id;
+  frame_cb_t		   frame_cb;
+  gpointer		   frame_cb_data;
+
+  guint			   end_time_id;
+  gint			   end_time; /* for triggering early "playback finished" */
 };
 
 static int gtv_table_signals[LAST_SIGNAL] = { 0 };
@@ -229,10 +248,13 @@ static inline gboolean is_fullscreen (gtk_video_private_t *priv)
 
 static gboolean gtv_test_inhibit_blank (GtkVideo *gtv)
 {
-  /* (fullscreen || (deny blank && drawable && playing)) && !logo && !paused */
+  /* (fullscreen || (deny blank && drawable && playing && !(shaded || hidden)))
+   * && !logo && !paused
+   */
   return (is_fullscreen (gtv->priv)
 	  || (gtv->priv->playing_no_blank && GTK_WIDGET_DRAWABLE (&gtv->widget)
-	      && xine_get_status (gtv->priv->stream) == XINE_STATUS_PLAY))
+	      && xine_get_status (gtv->priv->stream) == XINE_STATUS_PLAY
+	      && (gdk_window_get_state (gtk_widget_get_toplevel (&gtv->widget)->window) & 3) == 0))
 	 && !playlist_showing_logo ()
 	 && xine_get_param (gtv->priv->stream, XINE_PARAM_SPEED);
 }
@@ -509,8 +531,23 @@ static void gtv_do_resize (GtkVideo *gtv, const GtkRequisition *size)
   gtk_widget_set_size_request (&gtv->widget, size->width, size->height);
   if (GTK_WIDGET_REALIZED(&gtv->widget) && gtv->widget.parent)
   {
+    GdkGeometry geom;
+    GtkWindow *toplevel = GTK_WINDOW (gtk_widget_get_toplevel (&gtv->widget));
+    if (toplevel->bin.child == &gtv->widget)
+    {
+      geom.min_width = size->width;
+      geom.min_height = size->height;
+      gtk_window_set_geometry_hints (toplevel, &gtv->widget, &geom,
+				     GDK_HINT_MIN_SIZE);
+    }
     gtk_window_resize ((GtkWindow *)gtk_widget_get_toplevel (&gtv->widget),
-		       GTK_VIDEO_MIN_WIDTH, 1); /* note controlling geometry */
+		       size->width, size->height); /* note controlling geometry */
+    if (toplevel->bin.child == &gtv->widget)
+    {
+      geom.min_aspect = geom.max_aspect = (double)size->width / (double)size->height;
+      gtk_window_set_geometry_hints (toplevel, &gtv->widget, &geom,
+				     GDK_HINT_ASPECT);
+    }
     gtk_video_allow_shrink (gtv);
   }
 }
@@ -561,11 +598,12 @@ static void dest_size_cb (void *gv_gen,
   *dest_pixel_aspect = priv->display_ratio;
 }
 
-static gboolean gtk_video_idle_resize (GtkVideo *gtv)
+static gboolean gtv_idle_resize (GtkVideo *gtv, gboolean resize)
 {
   gdk_threads_enter ();
   RESIZE_LOCK ();
 
+  gboolean send_signal = FALSE;
   gtk_video_private_t *priv = gtv->priv;
   double factor = DOUBLE_STD();
   priv->size = SCALE (factor);
@@ -580,15 +618,26 @@ static gboolean gtk_video_idle_resize (GtkVideo *gtv)
   }
   else if (!priv->pending_resize)
   {
-    gtv_do_resize (gtv, &priv->size);
+    if (resize)
+      gtv_do_resize (gtv, &priv->size);
+    else
+      send_signal = gtv_update_resize_factor (gtv);
     logprintf ("gtkvideo: idle signal done\n");
   }
   else
   {
-    gtv_do_rescale (gtv);
+    if (resize)
+      gtv_do_rescale (gtv);
+    else
+      send_signal = gtv_update_resize_factor (gtv);
     logprintf ("gtkvideo: idle signal done, pending resize handled\n");
   }
 
+  if (send_signal)
+    g_signal_emit ((GObject *)gtv,
+		   gtv_table_signals[GTK_VIDEO_SCALE_FACTOR], 0,
+		   priv->resize_factor);
+
   priv->resizing = 0;
   priv->idle_resized = TRUE;
 
@@ -598,6 +647,38 @@ static gboolean gtk_video_idle_resize (GtkVideo *gtv)
   return FALSE;
 }
 
+static gboolean gtk_video_idle_resize (GtkVideo *gtv)
+{
+  return gtv_idle_resize (gtv, TRUE);
+}
+
+static gboolean gtk_video_idle_no_resize (GtkVideo *gtv)
+{
+  return gtv_idle_resize (gtv, FALSE);
+}
+
+static gboolean end_time_check (gtk_video_private_t *priv)
+{
+  gint pos_stream, pos_time, length_time;
+  priv->end_time_id = 0;
+  if (!xine_get_pos_length (stream, &pos_stream, &pos_time, &length_time))
+    pos_stream = pos_time = length_time = 0;
+  if (priv->end_time && pos_time >= priv->end_time)
+  {
+    logprintf ("gtkvideo: end time passed, sending stop event\n");
+    priv->end_time = 0;
+    xine_stream_t *stream = priv->stream;
+    xine_set_param (stream, XINE_PARAM_SPEED, XINE_SPEED_PAUSE);
+    xine_event_t event = {
+      .type = XINE_EVENT_UI_PLAYBACK_FINISHED,
+      .stream = stream,
+      .data = NULL
+    };
+    xine_event_send (stream, &event);
+  }
+  return FALSE;
+}
+
 static void frame_output_cb (void *gv_gen,
 			     int video_width, int video_height,
 			     double video_pixel_aspect,
@@ -624,7 +705,6 @@ static void frame_output_cb (void *gv_gen,
   priv->video_size = (GtkRequisition){ video_width, video_height };
   *dest_y = *dest_x = 0;
 
-  if (first || priv->auto_resize)
   {
     RESIZE_LOCK ();
     {
@@ -642,7 +722,7 @@ static void frame_output_cb (void *gv_gen,
           priv->pending_resize = TRUE;
         }
         else
-	  RESIZE_ADD ();
+	  RESIZE_ADD (first || priv->auto_resize);
       }
     }
     RESIZE_UNLOCK ();
@@ -665,25 +745,104 @@ static void frame_output_cb (void *gv_gen,
   }
 
   *dest_pixel_aspect = priv->display_ratio;
+
+   if (priv->end_time && !priv->end_time_id)
+     priv->end_time_id = g_idle_add ((GSourceFunc) end_time_check, priv);
+
+  if (priv->frame_cb)
+  {
+    if (priv->frame_cb_id)
+      g_source_remove (priv->frame_cb_id);
+    priv->frame_cb_id = g_idle_add (priv->frame_cb, priv->frame_cb_data);
+  }
 }
 
+#ifdef HAVE_XCB
+# ifdef ENABLE_X11_VO
+static xine_video_port_t *
+gxine_open_video_driver (xine_t * self, const char *id, void *xcb, void *x11)
+{
+  return xine_open_video_driver (self, id, XINE_VISUAL_TYPE_XCB, xcb)
+     ? : xine_open_video_driver (self, id, XINE_VISUAL_TYPE_X11, x11);
+}
+# else /* ! ENABLE_X11_VO */
+#  define gxine_open_video_driver(SELF,ID,XCB,X11) \
+  xine_open_video_driver ((SELF), (ID), XINE_VISUAL_TYPE_XCB, (XCB))
+# endif
+#else /* ! HAVE_XCB */
+# define gxine_open_video_driver(SELF,ID,X11,UNUSED) \
+  xine_open_video_driver ((SELF), (ID), XINE_VISUAL_TYPE_X11, (X11))
+#endif
+
 static xine_video_port_t *load_video_out_driver (GtkVideo *this)
 {
   double                   res_h, res_v;
-  x11_visual_t             vis;
+  int                      mm_h, mm_v, mm;
   const char              *video_driver_id;
   xine_video_port_t       *video_port;
   gtk_video_private_t     *priv = this->priv;
 
+#ifndef HAVE_XCB
+  x11_visual_t vis;
+
   vis.display           = gtv_get_xdisplay (this);
   vis.screen            = gtv_get_xscreen_num (this);
   vis.d                 = priv->video_window;
-  res_h                 = (DisplayWidth  (vis.display, vis.screen)*1000
-			   / DisplayWidthMM (vis.display, vis.screen));
-  res_v                 = (DisplayHeight (vis.display, vis.screen)*1000
-			   / DisplayHeightMM (vis.display, vis.screen));
-  priv->display_ratio   = res_v / res_h;
+  mm_h			= DisplayWidthMM (vis.display, vis.screen);
+  res_h			= DisplayWidth (vis.display, vis.screen);
+  mm_v			= DisplayHeightMM (vis.display, vis.screen);
+  res_v			= DisplayHeight (vis.display, vis.screen);
+#else /* HAVE_XCB */
+  xcb_visual_t vis;
+  int screen_no = 0;
+  xcb_connection_t *connection = xcb_connect(NULL, &screen_no);
+  xcb_screen_iterator_t screen_it;
 
+#ifdef ENABLE_X11_VO
+  x11_visual_t x11vis;
+#endif
+
+  screen_it = xcb_setup_roots_iterator (xcb_get_setup (connection));
+  while (screen_it.rem > 1 && screen_no > 0)
+  {
+    xcb_screen_next (&screen_it);
+    --screen_no;
+  }
+
+  vis.connection        = connection;
+  vis.screen            = screen_it.data;
+  vis.window            = priv->video_window;
+
+#ifdef ENABLE_X11_VO
+  x11vis.display        = gtv_get_xdisplay (this);
+  x11vis.screen         = gtv_get_xscreen_num (this);
+  x11vis.d              = priv->video_window;
+#endif
+
+  /* why can't the 'Merkians spell "millimetre" correctly... */
+  mm_h			= screen_it.data->width_in_millimeters;
+  res_h			= screen_it.data->width_in_pixels;
+  mm_v			= screen_it.data->height_in_millimeters;
+  res_v			= screen_it.data->height_in_pixels;
+#endif /* HAVE_XCB */
+
+  char *desc = g_strdup_printf (_("The reported value is %dmm."), mm_h);
+  mm = xine_config_register_range (priv->xine, "video.display_width",
+				   0, 0, 100000,
+				   N_("display width in mm\n(0 = autodetect)"), desc,
+				   10, NULL, NULL);
+  res_h = res_h * 1000 / (mm ? mm : mm_h);
+  free (desc);
+
+  desc = g_strdup_printf (_("The reported value is %dmm."), mm_v);
+  mm = xine_config_register_range (priv->xine, "video.display_height",
+				   0, -1, 100000,
+				   N_("display height in mm\n(0 = autodetect, -1 = assume square pixels)"), desc,
+				   10, NULL, NULL);
+  res_v = mm < 0 ? res_h : (res_v * 1000 / (mm ? mm : mm_v));
+  free (desc);
+
+  priv->display_ratio   = (res_h <= 0.0 || res_v <= 0.0) ? 1 : res_v / res_h;
   if (fabs(priv->display_ratio - 1.0) < 0.01)
     priv->display_ratio   = 1.0;
 
@@ -691,35 +850,46 @@ static xine_video_port_t *load_video_out_driver (GtkVideo *this)
   vis.frame_output_cb   = frame_output_cb;
   vis.user_data         = this;
 
-  if (priv->video_driver_id)
-    video_driver_id = priv->video_driver_id;
-  else
-  {
-    char **choices = get_driver_ids (xine_list_video_output_plugins (priv->xine));
-    /* try to init video with stored information */
-    int i = xine_config_register_enum (priv->xine,
-				   "video.driver", 0,
-				   choices,
-				   N_("video driver to use"),
-				   NULL, 10, NULL, NULL);
-    video_driver_id = choices[i];
-  }
+#ifdef ENABLE_X11_VO
+  x11vis.dest_size_cb    = dest_size_cb;
+  x11vis.frame_output_cb = frame_output_cb;
+  x11vis.user_data       = this;
+#endif
+
+  char **choices = get_driver_ids (gxine_list_video_output_plugins (priv->xine));
+  /* try to init video with stored information */
+  int i = xine_config_register_enum (priv->xine,
+				     "video.driver", 0, choices,
+				     N_("video driver to use"),
+				     NULL, 10, NULL, NULL);
+  video_driver_id = priv->video_driver_id ? : choices[i];
+
   if (strcmp (video_driver_id, "auto"))
   {
-    video_port=xine_open_video_driver (priv->xine,
-				      video_driver_id,
-				      XINE_VISUAL_TYPE_X11,
-				      (void *) &vis);
+    video_port = gxine_open_video_driver (priv->xine, video_driver_id,
+					  (void *) &vis, (void *) &x11vis);
     if (video_port)
       return video_port;
-    else
-      g_printerr (_("gtkvideo: video driver %s failed.\n"),
+
+    g_printerr (_("gtkvideo: video driver %s failed.\n"),
 	       video_driver_id); /* => auto-detect */
   }
 
-  return xine_open_video_driver (priv->xine, NULL,
-				 XINE_VISUAL_TYPE_X11,
-				 (void *) &vis);
+  video_port = gxine_open_video_driver (priv->xine, NULL,
+					(void *) &vis, (void *) &x11vis);
+
+  if (video_port)
+    return video_port;
+
+  g_printerr (_("gtkvideo: video driver %s failed.\n"), "auto");
+  return gxine_open_video_driver (priv->xine, "none",
+				  (void *) &vis, (void *) &x11vis);
+}
+
+uint32_t gtk_video_get_capabilities (GtkVideo *this)
+{
+  gtk_video_private_t *priv = this->priv;
+  return priv->video_port->get_capabilities (priv->video_port);
 }
 
 static void gtv_send_xine_mouse_event (gtk_video_private_t *priv,
@@ -761,8 +931,25 @@ static GdkFilterReturn xevent_filter_cb (XEvent *const xev, GdkEvent *const ev,
   {
   case Expose:
     if (!xev->xexpose.count)
+    {
+#ifndef HAVE_XCB
       xine_port_send_gui_data (priv->video_port,
 			       XINE_GUI_SEND_EXPOSE_EVENT, xev);
+#else
+      xcb_expose_event_t xcb_event;
+      memset(&xcb_event, 0, sizeof(xcb_event));
+
+      xcb_event.window = xev->xexpose.window;
+      xcb_event.x      = xev->xexpose.x;
+      xcb_event.y      = xev->xexpose.y;
+      xcb_event.width  = xev->xexpose.width;
+      xcb_event.height = xev->xexpose.height;
+      xcb_event.count  = xev->xexpose.count;
+
+      xine_port_send_gui_data (priv->video_port,
+			       XINE_GUI_SEND_EXPOSE_EVENT, &xcb_event);
+#endif
+    }
     logprintf ("gtkvideo: expose event: %d,%d %dx%d %d\n",
 	       xev->xexpose.x, xev->xexpose.y,
 	       xev->xexpose.width, xev->xexpose.height, xev->xexpose.count);
@@ -1040,8 +1227,17 @@ static void gtk_video_realize (GtkWidget *widget)
 					    widget->allocation.height, 0,
 					    black_pixel.pixel, black_pixel.pixel);
 
+  /* not sure why this is needed in some situations... */
+  XEvent event;
+  XCheckWindowEvent (xdisplay, priv->video_window, -1, &event);
+      
   widget->window = gdk_window_foreign_new_for_display
 		     (display, priv->video_window);
+  if (!widget->window)
+  {
+    g_printerr (_("gtkvideo: couldn't get a GDK handle for the video window!\n"));
+    return;
+  }
   gdk_window_set_user_data (widget->window, widget); /* allow GTK events */
   {
     /* enforce background colour */
@@ -1097,11 +1293,11 @@ static void gtk_video_realize (GtkWidget *widget)
    */
 
   priv->video_port = load_video_out_driver (this);
-
   if (!priv->video_port)
   {
-    g_printerr (_("gtkvideo: couldn't open video driver\n"));
-    return;
+    display_error_modal (FROM_GXINE, _("Fatal error"),
+			 _("No video output driver could be loaded."));
+    exit (2);
   }
 
   /*
@@ -1199,6 +1395,7 @@ GtkWidget *gtk_video_new (xine_t *xine, xine_stream_t *stream,
   priv->post_deinterlace.list = NULL;
   priv->post_video.list = NULL;
   priv->post_audio.list = NULL;
+  priv->frame_cb = NULL;
 
   if (video_driver_id)
     priv->video_driver_id = strdup (video_driver_id);
@@ -1210,6 +1407,61 @@ GtkWidget *gtk_video_new (xine_t *xine, xine_stream_t *stream,
   return this;
 }
 
+void gtk_video_set_frame_cb (GtkVideo *gtv, gtk_video_frame_cb_t cb,
+			     gpointer data)
+{
+   g_return_if_fail (GTK_IS_VIDEO(gtv));
+   gtv->priv->frame_cb = cb;
+   gtv->priv->frame_cb_data = data;
+}
+
+void gtk_video_set_end_time (GtkVideo *gtv, gint end_time)
+{
+   g_return_if_fail (GTK_IS_VIDEO(gtv));
+   gtv->priv->end_time = end_time;
+}
+
+static gboolean gtv_update_resize_factor (GtkVideo *this)
+{
+  gboolean send_signal = FALSE;
+  gtk_video_private_t *const priv = this->priv;
+  double new_factor;
+  if (priv->pending_resize)
+    new_factor = priv->resize_factor;
+  else
+  {
+    gboolean is_double =
+      IS_DOUBLE(priv->video_size.width, priv->video_size.height);
+    new_factor = this->widget.allocation.width * 100
+		 / (double)priv->video_size.width / (1 + is_double);
+    double new_factor_V = this->widget.allocation.height * 100
+			  / (double)priv->video_size.height
+			  / (1 + is_double);
+    if (new_factor > new_factor_V)
+      new_factor= new_factor_V;
+  }
+  /* tolerate rounding error when the video size has been changed... */
+  if (priv->pending_resize
+      || (round (priv->resize_factor * 64) != round (new_factor * 64)
+	  && (fabs (new_factor - priv->resize_factor)
+		>= 25 / this->widget.allocation.height
+	      || priv->old_video_size.height == priv->video_size.height)))
+  {
+    if (!priv->block_next_scale_change && GTK_WIDGET_VISIBLE (&this->widget))
+    {
+      priv->resize_factor = new_factor;
+      send_signal = TRUE;
+      /* stop frame_output_cb from scheduling a resize */
+      double factor = DOUBLE_STD();
+      priv->size = SCALE (factor);
+      logprintf ("gxine: video rescaled to %lf%%\n", new_factor);
+    }
+    else
+      logprintf ("blocked rescaling to %lf%%\n", new_factor);
+  }
+  return send_signal;
+}
+
 static void gtk_video_size_allocate (GtkWidget *widget,
     				     GtkAllocation *allocation)
 {
@@ -1238,43 +1490,7 @@ static void gtk_video_size_allocate (GtkWidget *widget,
 	&& GTK_WIDGET_VISIBLE (widget)
 	&& widget->allocation.width > GTK_VIDEO_MIN_WIDTH
 	&& widget->allocation.height > GTK_VIDEO_MIN_HEIGHT)
-    {
-      gtk_video_private_t *const priv = this->priv;
-      double new_factor;
-      if (priv->pending_resize)
-	new_factor = priv->resize_factor;
-      else
-      {
-	gboolean is_double =
-	  IS_DOUBLE(priv->old_video_size.width, priv->old_video_size.height);
-	new_factor = this->widget.allocation.width * 100
-		     / (double)priv->old_video_size.width / (1 + is_double);
-	double new_factor_V = this->widget.allocation.height * 100
-			      / (double)priv->old_video_size.height
-			      / (1 + is_double);
-	if (new_factor > new_factor_V)
-	  new_factor= new_factor_V;
-      }
-      /* tolerate rounding error when the video size has been changed... */
-      if (priv->pending_resize
-	  || (round (priv->resize_factor * 64) != round (new_factor * 64)
-	      && (fabs (new_factor - priv->resize_factor)
-		    >= 25 / this->widget.allocation.height
-		  || priv->old_video_size.height == priv->video_size.height)))
-      {
-	if (!priv->block_next_scale_change && GTK_WIDGET_VISIBLE (widget))
-	{
-	  priv->resize_factor = new_factor;
-	  send_signal = TRUE;
-	  /* stop frame_output_cb from scheduling a resize */
-	  double factor = DOUBLE_STD();
-	  priv->size = SCALE (factor);
-	  logprintf ("gxine: video rescaled to %lf%%\n", new_factor);
-	}
-        else
-	  logprintf ("blocked rescaling to %lf%%\n", new_factor);
-      }
-    }
+      send_signal = gtv_update_resize_factor (this);
     if (!this->priv->resizing)
       this->priv->old_video_size = this->priv->video_size;
     this->priv->idle_resized = FALSE;
@@ -1955,7 +2171,7 @@ void gtk_video_set_auto_rescale (GtkVideo *gtv, gboolean set)
 
   priv->auto_rescale = set;
 
-  RESIZE_ADD ();
+  RESIZE_ADD (1);
 }
 
 gboolean gtk_video_get_auto_rescale (GtkVideo *gtv)
@@ -2044,7 +2260,7 @@ static gboolean gtv_unblank_screen (GtkVideo *gtv)
     {
       static const char *const argv[] = { GNOME_SCREENSAVER_COMMAND, "--poke", NULL };
       /* screensaver name is used in "can't poke <name>" */
-      gnome_screensaver = screensaver_poke (argv, _("gnome-screensaver"));
+      gnome_screensaver = screensaver_poke (argv, "gnome-screensaver");
     }
 #endif
 
@@ -2056,7 +2272,7 @@ static gboolean gtv_unblank_screen (GtkVideo *gtv)
     if (!x_screensaver)
     {
       static const char *const argv[] = { X_SCREENSAVER_COMMAND, "--deactivate", NULL };
-      x_screensaver = screensaver_poke (argv, _("xscreensaver"));
+      x_screensaver = screensaver_poke (argv, "xscreensaver");
     }
 #endif
   }
