@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2001-2005 the xine-project
+ * Copyright (C) 2001-2008 the xine-project
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -16,8 +16,6 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
  * USA
  *
- * $Id: utils.c,v 1.74 2006/04/09 14:40:12 dsalt Exp $
- *
  * needful things
  */
 
@@ -47,6 +45,8 @@
 #include "playlist.h"
 #include "gtkvideo.h"
 
+int gxine_init_count = 0;
+
 static int ensure_mkdir (const char *path, mode_t mode)
 {
   struct stat st;
@@ -76,10 +76,127 @@ int ensure_path_exists (char *path, int mode)
   return ensure_mkdir (path, mode); /* leave mode as is if already present */
 }
 
+static char *filename_to_mrl_type (const char *path, const char *type)
+{
+  /* assumption: type looks like "foo:" and is <= 6 characters long */
+  size_t type_len = strlen (type);
+  char *mrl = g_filename_to_uri (path, NULL, NULL);
+  memmove (mrl + type_len, mrl + 6, strlen (mrl + 5));
+  memcpy (mrl, type, type_len);
+  return mrl;
+}
+
+static char *filename_to_dvd (const char *srcpath, struct stat *pst)
+{
+  struct stat st;
+  char *path = strdup (srcpath);
+  size_t length = strlen (path);
+  char *mrl, *vts = NULL;
+
+  if (!pst->st_mode)
+  {
+    /* initial stat failed, so see if we have .../title[.chapter] */
+    mrl = strrchr (path, '/'); /* can't fail */
+    int count = 0;
+    if ((sscanf (mrl, "/%*[0-9].%*[0-9]%n", &count), !mrl[count]) ||
+	(sscanf (mrl, "/%*[0-9]%n", &count), !mrl[count]))
+    {
+      *mrl = 0;
+      length = mrl - path;
+      if (stat (path, pst))
+	pst->st_mode = 0;
+    }
+  }
+
+  if (!S_ISDIR (pst->st_mode))
+  {
+    if (length >= 4 && !strcasecmp (path + length - 4, ".iso"))
+      mrl = filename_to_mrl_type (srcpath, "dvd:");
+    else
+      mrl = NULL;
+    free (path);
+    return mrl;
+  }
+
+  while (--length && G_IS_DIR_SEPARATOR (path[length]))
+    /**/;
+  path[length + 1] = 0;
+
+  vts = g_build_path (G_DIR_SEPARATOR_S, path, "VIDEO_TS", NULL);
+  if (!stat (vts, &st) && S_ISDIR (st.st_mode))
+    goto is_dvd;
+
+  strcpy (vts + strlen (vts) - 8, "video_ts"); /* overwrite, lower case */
+  if (!stat (vts, &st) && S_ISDIR (st.st_mode))
+  {
+  is_dvd:
+    free (vts);
+    free (path);
+    return filename_to_mrl_type (srcpath, "dvd:");
+  }
+
+  return NULL;
+}
+
+static char *filename_to_vcd (const char *srcpath, struct stat *st)
+{
+  char *mrl, *path = strdup (srcpath);
+
+  if (!st->st_mode)
+  {
+    /* initial stat failed, so see if we have ...@[EPST]?track */
+    mrl = strrchr (path, '@'); /* can fail */
+    if (mrl)
+    {
+      int count = 0;
+      if ((sscanf (mrl, "@%*1[EePpSsTt]%*u%n", &count), !mrl[count]) ||
+	  (sscanf (mrl, "@%*u%n", &count), !mrl[count]))
+      {
+	*mrl = 0;
+	if (stat (path, st))
+	  st->st_mode = 0;
+      }
+    }
+  }
+
+  if (S_ISDIR (st->st_mode))
+  {
+    free (path);
+    return NULL;
+  }
+
+  const char *suffix = strrchr (path, '.');
+  if (suffix && (!strcasecmp (suffix, ".cue") || !strcasecmp (suffix, ".nrg")))
+  {
+    free (path);
+    return filename_to_mrl_type (srcpath, "vcd:");
+  }
+
+  free (path);
+  return NULL;
+}
+
+static char *filename_to_uri (const char *path)
+{
+  struct stat st;
+  char *mrl;
+
+  if (stat (path, &st))
+    st.st_mode = 0; /* invalid */
+
+  if ((mrl = filename_to_dvd (path, &st)))
+    return mrl;
+
+  if ((mrl = filename_to_vcd (path, &st)))
+    return mrl;
+
+  return g_filename_to_uri (path, NULL, NULL);
+}
+
 char *make_path_uri (const char *path)
 {
   if (g_path_is_absolute (path))
-    return g_filename_to_uri (path, NULL, NULL);
+    return filename_to_uri (path);
 
   if ((*path & 0xDF) < 'A' || (*path & 0xDF) > 'Z')
     goto make_abs;
@@ -96,7 +213,7 @@ char *make_path_uri (const char *path)
   char *dir = g_get_current_dir ();
   char *abspath = g_build_path (G_DIR_SEPARATOR_S, dir, path, NULL);
   free (dir);
-  dir = g_filename_to_uri (abspath, NULL, NULL);
+  dir = filename_to_uri (abspath);
   free (abspath);
   return dir;
 }
@@ -166,7 +283,9 @@ char *read_entire_file (const char *mrl, ssize_t *file_size)
 
 char *get_config_filename (const char *leaf)
 {
-  return g_build_filename (g_get_home_dir (), ".gxine", leaf, NULL);
+  const char *cfgdir = g_get_user_config_dir ();
+  int home_is_not_cfgdir = !!strcmp (g_get_home_dir (), cfgdir);
+  return g_build_filename (cfgdir, ".gxine" + home_is_not_cfgdir, leaf, NULL);
 }
 
 FILE *open_write (const char *file, const char *errtitle)
@@ -213,6 +332,17 @@ void window_show (GtkWidget *const widget, GtkWidget *const parent)
   gdk_window_raise (widget->window);
 }
 
+void window_present (GtkWidget *const widget, GtkWidget *const parent)
+{
+  g_return_if_fail (widget != NULL);
+  g_return_if_fail (GTK_IS_WINDOW (widget));
+  g_return_if_fail (parent == NULL || GTK_IS_WINDOW (parent));
+  gtk_window_set_transient_for ((GtkWindow *)widget,
+				(GtkWindow *)(parent ? : app));
+  gtk_widget_show (widget);
+  gdk_window_raise (widget->window);
+}
+
 static void response_cb (GtkWidget *dialog, gint response, gpointer data)
 {
   free (g_object_get_data (G_OBJECT (dialog), "msg"));
@@ -265,8 +395,8 @@ void v_display_message (gxine_msg_source src, gboolean modal,
 					   (modal ? GTK_DIALOG_MODAL : 0), type,
 				   GTK_BUTTONS_CLOSE, "%s", primary ? : msg);
   if (primary)
-    gtk_message_dialog_format_secondary_markup (GTK_MESSAGE_DIALOG (dialog),
-						"%s", msg);
+    gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
+					      "%s", msg);
   gtk_window_set_title (GTK_WINDOW (dialog), gettext (wtitle[src][type]));
   gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_CENTER);
 
@@ -340,11 +470,15 @@ int parse_timestring (const char *string_time)
   int hours = 0, minutes = 0;
   double seconds = 0;
 
+  int sign = (*string_time == '-') ? -1 : 1;
+  if (sign < 0)
+    ++string_time;
+
   if (sscanf (string_time, "%d:%d:%lf", &hours, &minutes, &seconds) == 3)
-    return hours * 3600000 + minutes * 60000 + seconds * 1000;
+    return sign * (hours * 3600000 + minutes * 60000 + seconds * 1000);
 
   if (sscanf (string_time, "%d:%lf", &minutes, &seconds) == 2)
-    return minutes * 60000 + seconds * 1000;
+    return sign * (minutes * 60000 + seconds * 1000);
 
   return 0;
 }
@@ -375,10 +509,6 @@ static void update_start_dir (GtkWidget *dbox)
   }
 }
 
-#if GXINE_GTK_OLDER_THAN(2,8,0)
-WEAK (void, gtk_file_chooser_set_do_overwrite_confirmation);
-#endif
-
 gchar *modal_file_dialog (const char *title, gboolean loading, gboolean local,
 			  gboolean uri, const gchar *pattern, GtkWidget *extra,
 			  GtkWidget *parent)
@@ -397,8 +527,7 @@ gchar *modal_file_dialog (const char *title, gboolean loading, gboolean local,
   gtk_file_chooser_set_local_only (ch, local);
   gtk_file_chooser_set_select_multiple (ch, FALSE);
   gtk_file_chooser_set_extra_widget (ch, extra);
-  if (GTK_HAVE (gtk_file_chooser_set_do_overwrite_confirmation, 2, 8, 0))
-    gtk_file_chooser_set_do_overwrite_confirmation (ch, TRUE);
+  gtk_file_chooser_set_do_overwrite_confirmation (ch, TRUE);
 
   set_start_dir (dbox);
 
@@ -559,6 +688,35 @@ const char *get_copyright_notice (void)
   static gchar *copyright = NULL;
   if (!copyright)
     copyright = g_strdup_printf (_("© 2002-%d the xine project team"),
-				 2006);
+				 2008);
   return copyright;
 }
+
+#include <xine/xine_internal.h>
+#include <xine/configfile.h>
+void config_update_default (const char *key, int value)
+{
+  /* warning: accesses xine internals */
+  cfg_entry_t *entry = xine->config->lookup_entry (xine->config, key);
+  if (entry)
+    entry->num_default = value;
+}
+
+/* List a subset of video plugins */
+
+extern const char *const *xine_list_video_output_plugins_typed (xine_t *, uint64_t)
+	__attribute__((__weak__));
+
+const char *const *gxine_list_video_output_plugins (xine_t *xine)
+{
+  int types = 0;
+#if !defined(HAVE_XCB) || defined(ENABLE_X11_VO)
+  types |= 1 << XINE_VISUAL_TYPE_X11;
+#endif
+#ifdef HAVE_XCB
+  types |= 1 << XINE_VISUAL_TYPE_XCB;
+#endif
+  return xine_list_video_output_plugins_typed
+	 ? xine_list_video_output_plugins_typed (xine, types)
+	 : xine_list_video_output_plugins (xine);
+}
