@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2001-2005 the xine-project
+ * Copyright (C) 2001-2007 the xine-project
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -16,13 +16,11 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
  * USA
  *
- * $Id: utils.h,v 1.43 2006/04/04 22:08:52 dsalt Exp $
- *
  * needful things
  */
 
-#ifndef HAVE_UTILS_H
-#define HAVE_UTILS_H
+#ifndef GXINE_UTILS_H
+#define GXINE_UTILS_H
 
 #include <gtk/gtk.h>
 #include <gdk/gdk.h>
@@ -85,6 +83,7 @@ GSList *modal_multi_file_dialog (const char *title, gboolean local,
 void get_mrl_from_filesystem (GtkWindow *, GtkEntry *);
 
 void window_show (GtkWidget *widget, GtkWidget *parent); /* NULL => app */
+void window_present (GtkWidget *widget, GtkWidget *parent); /* ditto */
 
 /* attach to delete-event signal -> hide widget & set *visible = FALSE */
 void hide_on_delete (GtkWidget *widget, gboolean *visible);
@@ -106,4 +105,19 @@ void do_pending_events (void);
 
 const char *get_copyright_notice (void);
 
+int gxine_init_count;
+static inline void gxine_init_add (GSourceFunc func, gpointer data)
+{
+  ++gxine_init_count;
+  gtk_init_add (func, data);
+}
+static inline void gxine_init_done (void)
+{
+  --gxine_init_count;
+}
+
+void config_update_default (const char *key, int value);
+
+const char *const *gxine_list_video_output_plugins (xine_t *xine);
+
 #endif
